/*
 * BSD Licence:
 * Copyright (c) 2001, 2002 Ben Houston [ ben@exocortex.org ]
 * Exocortex Technologies [ www.exocortex.org ]
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the <ORGANIZATION> nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */


#pragma once

using namespace System;
using namespace System::Drawing;
using namespace System::Collections;
using namespace System::IO;

//using namespace Exocortex;
//using namespace Exocortex::Geometry3D;


namespace Exocortex {

	namespace Cg {

		public __gc class CgContext;
		public __gc class CgProgram;
		public __gc class CgBind;

		public __gc class CgUtils {
		public:
			static bool	InitGLExtensions() {
				//Trace2::Push();
				// Get the entry points for the extensions
				int result = glh_init_extensions(
					"GL_ARB_texture_compression " \
					"GL_EXT_texture_compression_s3tc " \
					"GL_EXT_texture_filter_anisotropic " );
				//Trace2::Pop();
				return	( result != 0 );
			}
			static void	ShutdownGLExtensions() {
				//Trace2::Push();
				glh_shutdown_extensions();
				//Trace2::Pop();
			}

			static String*	GetCgCompilerPath() {
				char subKey[1024] = "SOFTWARE\\NVIDIA Corporation\\Global\\Cg\\CG_COMPILER_EXE\\";
				LONG result;
				HKEY hKey;

				// Create or open the registry key
				result = RegOpenKeyEx( HKEY_LOCAL_MACHINE, subKey, 0, KEY_READ, &hKey );
				if( result != ERROR_SUCCESS ) {
					return	NULL;//new String( "GetCgCompilerPath(): RegOpenKeyEx failed" );
				}
				//return	new String( "GetCgCompilerPath(): RegOpenKeyEx succeeded" );

				// Make room in the buffer to receive the incoming data.
				char buffer[1024];
				DWORD length = sizeof( buffer );
				result = RegQueryValueEx( hKey, NULL, 0, NULL, (LPBYTE) buffer, &length );
				if( result != ERROR_SUCCESS ) {
					return	NULL;//new String( "GetCgCompilerPath(): RegQueryValueEx failed" );
				}
				//OutputDebugString( "GetCgCompilerPath(): RegQueryValueEx succeeded" );

				// Close the registry key.
				RegCloseKey( hKey );
				//if( result != ERROR_SUCCESS ) {
				//	return	NULL;//new String( "GetCgCompilerPath(): RegCloseKey failed" );
				//}
				//OutputDebugString( "GetCgCompilerPath(): RegCloseKey succeeded" );

				//OutputDebugString( buffer );
				return	new String( buffer );
			}

			/*static String*	CompileCgFile( String* filePath, CgProfileType profileType, bool optimize, String* outputPath ) {
				if( File::Exists( filePath ) == false ) {
					throw new FileNotFoundException( filePath );
				}

				String* profileDescriptor = "";
				switch( profileType ) {
					case CgProfileType::Vertex:
						profileDescription = new String( "vp20" );
						break;
					case CgProfileType::Vertex30:
						profileDescription = new String( "vp30" );
						break;
					case CgProfileType::Fragment:
						profileDescription = new String( "fp30" );
						break;
					case CgProfileType::Fragment20:
						profileDescription = new String( "fp20" );
						break;
					case CgProfileType::ARBVertex:
						profileDescription = new String( "arbvp1" );
						break;
					default:
						throw new ArgumentException( String::Concat( "profile type not support: ", Enum::GetName( __typeof( CgProfileType ), __box( profileType ) ) );
				}

				String* commandLine = String::Concat( filePath, " ", "-profile", " ", profileDescriptor );
				if( optimize ) {
					commandLine = String::Concat( commandLine, " ", "-o" );
				}

				commandLine = String::Concat( commandLine, " ", outputPath );

				String* resultsFile = Path::ChangeExtension( filePath, ".log" );
				commandLine = String::Concat( commandLine, " ", ">>", " ", resultsFile );

				System::Console::
			}*/

			static void	Cleanup() {
				//Trace2::Push();
				// Final cleanup
				//Trace2::WriteLine( "cleaning up..." );
				cgCleanup();
				//Trace2::Pop();
			}

			static String*	GetErrorString( cgError error ) {
				int	cgErrorNumber = (int) error;
				switch( cgErrorNumber ) {
					case 0:
						return	"No error has occurred. (NoError)";
					case 1:
						return	"The compile returned an error. (CompileError)";
					case 2:
						return	"The parameter used is of invalid type. (InvalidParameterError)";
					case 3:
						return	"The profile is not supported. (InvalidProfileError)";
					case 4:
						return	"The program did could not load. (ProgramLoadError)";
					case 5:
						return	"The program could not bind. (ProgramBindError)";
					case 6:
						return	"The program must be loaded before this operation may be used. (ProgramNotLoadedError)";
					case 7:
						return	"An unsupported GL extension was required to perform this operation. (UnsupportedGLExtensionError)";
					case 8:
						return	"An unknown value type was assigned to a parameter. (InvalidValueTypeError)";
					case 9:
						return	"The parameter is not of matrix type. (NotMatrixParamError)";
					case 10:
						return	"The enumerant parameter has an invalid value. (InvalidEnumerantError)";
					case 11:
						return	"The parameter must be a 4x4 matrix type. (Not4x4MatrixError)";
					case 12:
						return	"The file could not be read. (FileReadError)";
					case 13:
						return	"The file could not be written. (FileWriteError)";
					default:
						return	String::Concat( "Unknown error code (", cgErrorNumber.ToString(), ")" );
				}
			}
		};

		public __gc class CgException : public Exception {
		public:
			CgException( cgError error ) : Exception( CgUtils::GetErrorString( error ) ) {
				_error = error;
			}
			CgException( String* message ) : Exception( message ) {
				_error = (cgError) 0;
			}

			cgError	GetCGError() { 
				return	_error;
			}

		protected:
			cgError	_error;
		};

		public __value enum	CgProfileType {
			Vertex		= cgVertexProfile,		//CG_PROFILE_MACRO(Vertex,vp20,"vp20")
			Fragment20	= cgFragment20Profile,	//CG_PROFILE_MACRO(Fragment20,fp20,"dx8ps")
			Vertex30	= cgVertex30Profile,	//CG_PROFILE_MACRO(Vertex30,vp30,"vp30")
			Fragment	= cgFragmentProfile,	//CG_PROFILE_MACRO(Fragment,fp30,"fp30")
			ARBVertex	= cgARBVertexProfile	//CG_PROFILE_MACRO(ARBVertex,arbvp1,"arbvp1")
		};

		public __value enum	CgMatrixFormat {
			Identity	= cgGLMatrixIdentity,
			Transport	= cgGLMatrixTranspose,
			Inverse		= cgGLMatrixInverse,	
		};

		public __value enum	CgMatrixType {
			ModelView			= cgGLModelViewMatrix,
			Projection			= cgGLProjectionMatrix,
			Texture				= cgGLTextureMatrix,
			ModelViewProjection	= cgGLModelViewProjectionMatrix,
		};

		public __gc class CgBind {
		public:
			CgBind( CgProgram* program, cgProgramIter* pcgProgramIter, cgBindIter* pcgBindIter, String* name ) {
				//Trace2::Push();
				if( program == NULL ) {
					throw new ArgumentNullException( "program is null" );
				}
				_program = program;

				if( pcgProgramIter == NULL ) {
					throw new ArgumentNullException( "pcgProgramIter is null" );
				}
				_pcgProgramIter = pcgProgramIter;

				if( pcgBindIter == NULL ) {
					throw new ArgumentNullException( "pcgBindIter is null" );
				}
				_pcgBindIter = pcgBindIter;

				if( name == NULL ) {
					throw new ArgumentNullException( "name is null" );
				}
				_name = name;

				_clientStateEnabled = false;

				//Trace2::Pop();
			}
			~CgBind() {
				//Trace2::Push();
				Dispose();
				//Trace2::Pop();
			}

			void	Dispose() {
				//Trace2::Push();
				if( _pcgBindIter != NULL ) {
					cgFreeBindIter( _pcgBindIter );
					_pcgBindIter = NULL;
				}
				//Trace2::Pop();
			}	

			void	EnableClientState() {
				//Trace2::Push();

				// Enable client states
				//Trace2::WriteLine( "enabling client state..." );
				cgError error = cgGLEnableClientState( _pcgProgramIter, _pcgBindIter );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				_clientStateEnabled = true;
				//Trace2::Pop();
			}

			void	DisableClientState() {
				//Trace2::Push();

				// Disable client states
				//Trace2::WriteLine( "disabling client client state..." );
				cgError error = cgGLDisableClientState( _pcgProgramIter, _pcgBindIter );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				_clientStateEnabled = true;
				//Trace2::Pop();
			}

			void	BindUniform4f( float scalar ) {
				this->BindUniform4f( scalar, 0, 0, 0 );
			}
			void	BindUniform4f( Color c ) {
				this->BindUniform4f( c.R / 255.0F, c.G  / 255.0F, c.B / 255.0F, c.A / 255.0F );
			}
			void	BindUniform4f( float x, float y, float z ) {
				this->BindUniform4f( x, y, z, 1.0 );
			}
			void	BindUniform4f( float v0, float v1, float v2, float v3 ) {
				//Trace2::Push();
				cgError error = cgGLBindUniform4f( _pcgProgramIter, _pcgBindIter, v0, v1, v2, v3 );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}

			void	BindVaryingPointer( int elementSize, void *data ) {
				//Trace2::Push();
				if( elementSize <= 0 ) {
					throw new ArgumentOutOfRangeException( "elementSize", __box( elementSize ), "can not be <= 0" ); 
				}
				if( data == NULL ) {
					throw new ArgumentNullException( "data" ); 
				}
				cgError error = cgGLBindVaryingPointer( _pcgProgramIter, _pcgBindIter, elementSize, GL_FLOAT, 0, data );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}

			void	BindUniformStateMatrix( CgMatrixType matrixType, CgMatrixFormat matrixFormat ) {
				//Trace2::Push();
				cgError error = cgGLBindUniformStateMatrix( _pcgProgramIter, _pcgBindIter, (cgGLMatrixType) matrixType, (cgGLMatrixFormat) matrixFormat );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}

			__property CgProgram*		get_Program()				{	return	_program;	}
			__property cgBindIter*		get_Pointer()				{	return	_pcgBindIter;	}
			__property String*			get_Name()					{	return	_name;	}
			__property bool				get_ClientStateEnabled()	{	return	_clientStateEnabled;	}

		protected:
			CgProgram*		_program;
			cgProgramIter*	_pcgProgramIter;
			cgBindIter*		_pcgBindIter;
			String*			_name;
			bool			_clientStateEnabled;

		};

		public __gc class CgProgram {
		public:
			CgProgram( CgContext *context, cgProgramIter* pcgProgramIter, String* name ) {
				//Trace2::Push();
				if( context == NULL ) {
					throw new ArgumentNullException( "context is null" );
				}
				_context = context;
				
				if( pcgProgramIter == NULL ) {
					throw new ArgumentNullException( "pcgProgramIter is null" );
				}
				_pcgProgramIter = pcgProgramIter;

				if( name == NULL ) {
					throw new ArgumentNullException( "name is null" );
				}
				_name = name;
				//Trace2::Pop();
			}
			~CgProgram() {
				//Trace2::Push();
				Dispose();
				//Trace2::Pop();
			}

			void	Dispose() {
				//Trace2::Push();
				if( _pcgProgramIter != NULL ) {
					// Free program iterator
					//Trace2::WriteLine( "freeing program..." );
					cgFreeProgramIter( _pcgProgramIter );
					_pcgProgramIter = NULL;
				}
				//Trace2::Pop();
			}

			void	Load( int programId ) {
				//Trace2::Push();

				// Feed the compiled program to the graphics hardware
				//Trace2::WriteLine( "feed the compiled program to the graphics hardware..." );
				cgError error = cgGLLoadProgram( _pcgProgramIter, programId );
				if( error != cgNoError ) {
					throw new CgException( error );
				}

				_loaded = true;

				//Trace2::Pop();
			}

			void	Bind() {
				//Trace2::Push();
				// Bind vertex program
				cgError error = cgGLBindProgram( _pcgProgramIter );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}


			
			CgBind*		GetBindByName( String* bindName ) {
				//Trace2::Push();
				if( bindName == NULL ) {
					throw new ArgumentNullException( "bindName is null" );
				}
				
				//Trace2::WriteLine( String::Concat( "getting '", bindName, "' bind..." ) );

				if( this->Loaded == false ) {
					throw new CgException( "program not loaded" );
				}

				CString strBindName( bindName ); 
				cgBindIter* binding = cgGetBindByName( _pcgProgramIter, strBindName );
				if( binding == NULL ) {
					throw new ArgumentException( String::Concat( "could not get binding for: ", bindName ) );
				}

				//Trace2::Pop();

				return	new CgBind( this, _pcgProgramIter, binding, bindName );
			}

			__property CgContext*		get_Context()				{	return	_context;	}
			__property cgProgramIter*	get_Pointer()				{	return	_pcgProgramIter;	}
			//__property CgProfileType	get_ProfileType()			{	return	_profileType;	}
			__property String*			get_Name()					{	return	_name;	}
			__property bool				get_Loaded()				{	return	_loaded;	}
			
		protected:
			CgContext*		_context;
			cgProgramIter*	_pcgProgramIter;
			//CgProfileType	_profileType;
			System::String*	_name;
			bool			_loaded;
		};




		public __gc class CgContext {
		public:
			CgContext() {
				//Trace2::Push();
				// Create a context that will contain the Cg programs
				//Trace2::WriteLine( "creating cg context..." );
				_pcgContext = cgCreateContext();
				if( _pcgContext == NULL ) {
					throw new CgException( "cgCreateContext failed" );
				}
				//_htProgramProfiles = new Hashtable();
				//Trace2::Pop();
			}
			~CgContext() {
				//Trace2::Push();
				Dispose();
				//Trace2::Pop();
			}

			void	Dispose() {
				//Trace2::Push();
				if( _pcgContext != NULL ) {
					// Free context
					//Trace2::WriteLine( "freeing context..." );
					cgFreeContext( _pcgContext );
					_pcgContext = NULL;
				}
				//Trace2::Pop();
			}

			void	AddProgramFromFile( String* fileName, CgProfileType profileType ) {
				//Trace2::Push();
				if( File::Exists( fileName ) == false ) {
					throw new FileNotFoundException( String::Concat( "could not find file: ", fileName ) );
				}

				//Trace2::WriteLine( "loading and compiling cg source..." );
				CString strFileName( fileName ); 

				cgError error = cgAddProgramFromFile( _pcgContext, strFileName, (cgProfileType) profileType, 0 );
				if( error != cgNoError ) {
					throw new CgException( error );
				}

				//_htProgramProfiles->Add( fileName, __box( profileType ) );

				//Trace2::Pop();
			}

			CgProgram*		GetProgramByName( String* programName ) {
				//Trace2::Push();
				if( programName == NULL ) {
					throw new ArgumentNullException( "programName", programName );
				}

				//if( _htProgramProfiles->ContainsKey( programName ) == false ) {
				//	throw new CgException( String::Concat( "program not found: ", programName ) );
				//}

				// Get the program handle from the context based on the name of its entry point function
				//Trace2::WriteLine( "getting handle for program..." );
				CString strProgramName( programName ); 
				cgProgramIter *pcgProgramIter = cgProgramByName( _pcgContext, strProgramName );
				if( pcgProgramIter == NULL ) {
					throw new CgException( String::Concat( "cgProgramByName failed: ", programName ) );
				}
			
				//Object* pValue = _htProgramProfiles->get_Item( programName );
				//CgProfileType profileType = *dynamic_cast<__box CgProfileType*>( pValue ); 

				//Trace2::Pop();

				return	new CgProgram( this, pcgProgramIter, programName );
			}
			
			void	EnableProgramType( CgProfileType profileType ) {
				//Trace2::Push();
				// Enable profile type
				cgError error = cgGLEnableProgramType( (cgProfileType) profileType );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}
		
			
			// Disable the program after rendering
			void	DisableProgramType( CgProfileType profileType ) {
				//Trace2::Push();
				cgError error = cgGLDisableProgramType( (cgProfileType) profileType );
				if( error != cgNoError ) {
					throw new CgException( error );
				}
				//Trace2::Pop();
			}

			__property cgContext*	get_Pointer()				{	return	_pcgContext;	}

		protected:
			cgContext*		_pcgContext;
			//Hashtable*		_htProgramProfiles;		
		};


	}

}
