/*
 *
 * Copyright (c) 2002, NVIDIA Corporation.
 * 
 *  
 * 
 * NVIDIA Corporation("NVIDIA") supplies this software to you in consideration 
 * of your agreement to the following terms, and your use, installation, 
 * modification or redistribution of this NVIDIA software constitutes 
 * acceptance of these terms.  If you do not agree with these terms, please do 
 * not use, install, modify or redistribute this NVIDIA software.
 * 
 *  
 * 
 * In consideration of your agreement to abide by the following terms, and 
 * subject to these terms, NVIDIA grants you a personal, non-exclusive license,
 * under NVIDIAs copyrights in this original NVIDIA software (the "NVIDIA 
 * Software"), to use, reproduce, modify and redistribute the NVIDIA 
 * Software, with or without modifications, in source and/or binary forms; 
 * provided that if you redistribute the NVIDIA Software, you must retain the 
 * copyright notice of NVIDIA, this notice and the following text and 
 * disclaimers in all such redistributions of the NVIDIA Software. Neither the 
 * name, trademarks, service marks nor logos of NVIDIA Corporation may be used 
 * to endorse or promote products derived from the NVIDIA Software without 
 * specific prior written permission from NVIDIA.  Except as expressly stated 
 * in this notice, no other rights or licenses express or implied, are granted 
 * by NVIDIA herein, including but not limited to any patent rights that may be 
 * infringed by your derivative works or by other works in which the NVIDIA 
 * Software may be incorporated. No hardware is licensed hereunder. 
 * 
 *  
 * 
 * THE NVIDIA SOFTWARE IS BEING PROVIDED ON AN "AS IS" BASIS, WITHOUT 
 * WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
 * WITHOUT LIMITATION, WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT, 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR ITS USE AND OPERATION 
 * EITHER ALONE OR IN COMBINATION WITH OTHER PRODUCTS.
 * 
 *  
 * 
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
 * EXEMPLARY, CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOST 
 * PROFITS; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 * PROFITS; OR BUSINESS INTERRUPTION) OR ARISING IN ANY WAY OUT OF THE USE, 
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE NVIDIA SOFTWARE, 
 * HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING 
 * NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF NVIDIA HAS BEEN ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */ 

#ifndef _cg_h
#define _cg_h

/*************************************************************************/
/*** CG Run-Time Library API                                          ***/
/*************************************************************************/

/*************************************************************************/
/*** Data types and structures                                         ***/
/*************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
 {
# define CG_ERROR_MACRO(code, enum_name, message) \
   cg##enum_name = code,
# include <Cg/cg_errors.h>
 } cgError;

typedef struct
 {
  /* Opaque structure */
  void *Data;
 } cgContext;

typedef struct
 {
  /* Opaque structure */
  void *Data;
 } cgProgramIter;

typedef struct
 {
  /* Opaque structure */
  void *Data;
 } cgBindIter;

typedef enum
 {
# define CG_DATATYPE_MACRO(name, compiler_name, ncols, nrows) \
  cg##name##ValueType,

#include <Cg/cg_datatypes.h>

  cgUnknownValueType
 } cgValueType;

typedef enum
 {
  cgConstParam,
  cgUniformParam,
  cgConnectorMemberParam,
  cgVaryingParam = cgConnectorMemberParam,
  cgTexObjParam,
  cgAnyParam,
  cgUndefinedParam,
 } cgParamType;

typedef enum
 {

# define CG_BINDLOCATION_MACRO(name,compiler_name,\
                               enum_int,addressable,param_type) \
  cgBind##name = enum_int,
#include <Cg/cg_bindlocations.h>

  cgBindUndefined
 } cgBindLocation;


typedef enum
 {
# define CG_PROFILE_MACRO(name, compiler_id, compiler_opt) \
   cg##name##Profile,
# include "Cg/cg_profiles.h"

  cgNProfiles,
  cgUnknownProfile,
 } cgProfileType;

/*************************************************************************/
/*** Function Prototypes                                               ***/
/*************************************************************************/


/*******************************************************************/
/*** cg Misc Functions                                         ****/
/*******************************************************************/

extern void cgCleanup(void);

/* This will probably be deprecated in future releases */
extern void cgSetCompilerExe(const char *exe);

extern const char *cgErrorMsg(cgError error);

/*******************************************************************/
/*** cgContext Functions                                       ****/
/*******************************************************************/

extern cgContext *cgCreateContext(void);

extern void cgFreeContext(cgContext *context);

extern cgError cgAddProgram(cgContext *context,
                            const char *text,
                            cgProfileType profile,
                            const char *entry);

extern cgError cgAddProgramFromFile(cgContext *context,
                                    const char *filename,
                                    cgProfileType profile,
                                    const char *entry);

extern cgError cgAddProgramArgs(cgContext *context,
                                const char *text,
                                cgProfileType profile,
                                const char *entry,
                                int nargs,
                                const char **args);

extern cgError cgAddProgramFromFileArgs(cgContext *context,
                                        const char *filename,
                                        cgProfileType profile,
                                        const char *entry,
                                        int nargs,
                                        const char **args);

extern const char *cgGetLastListing(cgContext *context);

/*******************************************************************/
/*** cgProgram Functions                                        ***/
/*******************************************************************/

extern cgProgramIter *cgGetNextProgram(cgContext *context,
                                       cgProgramIter *iter);

extern cgProgramIter *cgProgramByName(cgContext *context, 
                                      const char *name);


extern cgProgramIter *cgGetLastProgram(cgContext *context);

extern void cgFreeProgramIter(cgProgramIter *program);

extern cgProgramIter *cgDuplicateProgram(cgProgramIter *program);

extern const char *cgGetProgramName(const cgProgramIter *program);

extern cgProfileType cgGetProgramProfile(const cgProgramIter *program);

extern const char *cgGetProgramObjectCode(const cgProgramIter *program);



/*******************************************************************/
/*** Parameter and Binding Functions                            ****/
/*******************************************************************/

extern cgBindIter *cgGetNextBind(cgProgramIter *program,
                                 cgBindIter *iter);

extern cgBindIter *cgGetBindByName(cgProgramIter *program, 
                                   const char *parameter_name);

extern cgBindIter *cgDuplicateBindIter(cgBindIter *bind);

extern void cgFreeBindIter(cgBindIter *iter);

extern const char *cgGetBindParamName(const cgBindIter *bind);

extern cgParamType cgGetBindParamType(const cgBindIter *bind);

extern cgBindLocation cgGetBindLocation(const cgBindIter *bind);

extern unsigned long cgGetBindAddress(const cgBindIter *bind);

extern cgValueType cgGetBindValueType(const cgBindIter *bind,
                                      int *array_size);

#ifdef __cplusplus
}
#endif


#endif
