using System;
using System.Diagnostics;



namespace Exocortex.Collections.Specialized
{
	/// <summary>
	/// Summary description for UInt40.
	/// </summary>
	public class UInt40 {

		protected const uint	cMask3Bits	= 0x7;
		protected const uint	cMask7Bits	= 0x3f;
		protected const ulong	cMask40Bits		= 0x0ffffffffffUL;
		protected const ulong cUInt40MaxValue	= 0x0ffffffffffUL;

		//-------------------------------------------------------------------------
		
		static protected int	Calc40To64Quotient( int index40 ) {
			return	( index40 * 5 ) / 8;//( index40 >> 1 ) + ( index40 >> 3 );
		}
		static protected int	Calc40To64Quotient_Control( int index40 ) {
			return	( index40 * 5 ) / 8;
		}

		static protected int	Calc40To64Remainder( int index40 ) {
			return	(int)(( index40 << 5 ) + ( index40 << 3 )) & (int)cMask7Bits;
		}
		static protected int	Calc40To64Remainder_Control( int index40 ) {
			return	( index40 * 40 ) % 64;
		}

		static protected int	ConvertSize40ToSize64( int size40 ) {
			return	Calc40To64Quotient( size40 ) + 1;
		}
		static protected int	ConvertSize64ToSize40( int size64 ) {
			return	(int)(( ((long)size64) << 3 ) / 5 ) - 1;
		}
		static protected int	ConvertSize64ToSize40_Control( int size64 ) {
			return	size64 * 8 / 5;
		}

		//-------------------------------------------------------------------------

		static public ulong[]	CreateArray( int size40 ) {
			Debug.Assert( 0 <= size40 );
			return	new	ulong[ Calc40To64Quotient( size40 ) + 2 ];
		}
		
		static public int	GetSize40( ulong[] data64 ) {
			Debug.Assert( data64 != null );
			return	ConvertSize64ToSize40( data64.Length );
		}

		static public int	GetSize64FromCount40( int count40 ) {
			return	Calc40To64Quotient( count40 ) + 1;
		}
	
		static public ulong	GetElement( ulong[] data64, int index40 ) {
			//Debug2.Push();

			Debug.Assert( data64 != null );
			Debug.Assert( ( 0 <= index40 ) && ( index40 < GetSize40( data64 ) ) );

			//Debug.WriteLine( "index40 = " + index40 );

			int	index64		= Calc40To64Quotient( index40 );
			int	remainder64	= Calc40To64Remainder( index40 );

			Debug.Assert( ( index64 + 1 ) < data64.Length );

			//Debug.WriteLine( "index64 = " + index64 );
			//Debug.WriteLine( "remainder64 = " + remainder64 );

			ulong	element40	= 0;

			//Debug.WriteLine( "data[index64,index64+1] = { " +
			//	Math2.ToHexadecimal( data64[ index64 ], 16 ) + " " +
			//	Math2.ToHexadecimal( data64[ index64 + 1 ], 16 ) + " }" );

			if( remainder64 < 24 ) {
				//Debug.WriteLine( "method 1" );
				element40 = ( data64[ index64 ] >> (int)( 24 - remainder64 ) ) & cMask40Bits;
			}
			else if( remainder64 == 24 ) {
				//Debug.WriteLine( "method 2" );
				element40 = data64[ index64 ] & cMask40Bits;
			}
			else { /* remainder > 24 */
				//Debug.WriteLine( "method 3" );
				element40 =
					(
					( data64[ index64 ] << (int)( remainder64 - 24 ) ) |
					( data64[ index64 + 1 ] >> (int)( 64 + 24 - remainder64 ) )
					) & cMask40Bits;
			}

			//Debug.WriteLine( "element40 = " + element40 + " [ 0x" + Math2.ToHexadecimal( element40, 10 ) + " ]" );
			
			Debug.Assert( ( element40 & cMask40Bits ) == element40 );

			//Debug2.Pop();

			return	element40;
		}

		static public void		SetElement( ulong[] data64, int index40, ulong element40 ) {
			//Debug2.Push();
			
			Debug.Assert( data64 != null );
			Debug.Assert( ( 0 <= index40 ) && ( index40 < GetSize40( data64 ) ) );
			Debug.Assert( ( element40 & cMask40Bits ) == element40 );
			
			//Debug.WriteLine( "index40 = " + index40 );
			
			int	index64		= Calc40To64Quotient( index40 );
			int	remainder64	= Calc40To64Remainder( index40 );

			Debug.Assert( ( index64 + 1 ) < data64.Length );

			//Debug.WriteLine( "index64 = " + index64 );
			//Debug.WriteLine( "remainder64 = " + remainder64 );
			
			//Debug.WriteLine( "element40 = " + element40 + " [ 0x" + Math2.ToHexadecimal( element40, 10 ) + " ]" );
			
			//Debug.WriteLine( "before  data[index64,index64+1] = { " +
			//	Math2.ToHexadecimal( data64[ index64 ], 16 ) + " " +
			//	Math2.ToHexadecimal( data64[ index64 + 1 ], 16 ) + " }" );
			
			if( remainder64 < 24 ) {
				//Debug.WriteLine( "method 1" );
				int shift = (int)( 24 - remainder64 );
				data64[ index64 ] =
					( data64[ index64 ] & ~ ( cMask40Bits << shift ) ) |
					( element40 << shift );
			}
			else if( remainder64 == 24 ) {
				//Debug.WriteLine( "method 2" );
				data64[ index64 ] =
					( data64[ index64 ] & ~ cMask40Bits ) |
					element40;
			}
			else {	/* remainder > 24 */
				//Debug.WriteLine( "method 3" );
				int shiftA = (int)( remainder64 - 24 );
				data64[ index64 ] = 
					( data64[ index64 ] & ~ ( cMask40Bits >> shiftA ) ) |
					( element40 >> shiftA );
				int	shiftB = (int)( 64 + 24 - remainder64 );
				data64[ index64 + 1 ] = 
					( data64[ index64 + 1 ] & ~ ( cMask40Bits << shiftB ) ) |
					( element40 << shiftB );
			}

			//Debug.WriteLine( "after   data[index64,index64+1] = { " +
			//	Math2.ToHexadecimal( data64[ index64 ], 16 ) + " " +
			//	Math2.ToHexadecimal( data64[ index64 + 1 ], 16 ) + " }" );
			
			/*int shiftAlpha = 24 - remainder64;
			data64[ index64 ] =	
				( data64[ index64 ] & ~ ( cMask40Bits << shiftAlpha ) ) |
				( element40 << shiftAlpha );
			if( shift < 0 ) {
				int	shiftBeta = ( 64 + 24 - remainder64 );
				data64[ index64 + 1 ] = 
					( data64[ index64 + 1 ] & ~ ( cMask40Bits << shiftBeta ) ) |
					( element40 << shiftBeta );
			}*/
			//Debug2.Pop();
		}
	}
}
