using System;
using System.Text;
using System.Collections;
using System.Reflection;
using System.Threading;
using System.Diagnostics;

namespace Exocortex.Diagnostics
{
	/// <summary>
	/// Summary description for ProfilingStack.
	/// </summary>
	public class ProfilingStack {
		
		//=======================================================================================

		private ProfilingStack() {}

		//==================================================================

		static private bool	_showTime = true;
		static public bool ShowTime {
			get	{	return	_showTime;	}
			set	{	_showTime = value;	}
		}
		
		static private bool	_showMemory = true;
		static public bool ShowMemory {
			get	{	return	_showMemory;	}
			set	{	_showMemory = value;	}
		}

		static private bool	_showBrackets = true;
		static public bool ShowBrackets {
			get	{	return	_showBrackets;	}
			set	{	_showBrackets = value;	}
		}

		//=======================================================================================

		protected class ProfilingFrame {
			protected DateTime	_datetimeBegin;
			protected String	_name;
			protected long		_memoryBegin;

			public ProfilingFrame( string name ) {
				_name = name;
				_memoryBegin = System.GC.GetTotalMemory( false );
				_datetimeBegin = System.DateTime.Now;
			}
			public float	GetElapsedMillisecs() {
				TimeSpan timespan = System.DateTime.Now - _datetimeBegin;
				return	(float) timespan.TotalMilliseconds;
			}
			public long		GetMemoryDelta() {
				long memoryDelta = System.GC.GetTotalMemory( false ) - _memoryBegin;
				return	memoryDelta;
			}
		}

		static protected Stack	_stackProfilingFrames = new Stack();

		//=======================================================================================

		static public string Push() {
			StackTrace st = new StackTrace( 1, true );
			StackFrame frame = st.GetFrame( 0 );
			MethodBase method = frame.GetMethod();

			string strFrameName = method.DeclaringType + "." + method.Name;
			return	ProfilingStack.Push( strFrameName );
		}

		static public string Push( String strName ) {
			lock( _stackProfilingFrames ) {
				ProfilingFrame frame = new ProfilingFrame( strName );
				_stackProfilingFrames.Push( frame );
				
				if( _showBrackets ) {
					return	strName + " {";
				}
				else {
					return	strName;
				}
			}
		}

		static public string Pop() {
			lock( _stackProfilingFrames ) {
				Debug.Assert( _stackProfilingFrames.Count > 0 );
				ProfilingFrame frame = (ProfilingFrame) _stackProfilingFrames.Pop();

				float fMilliseconds = 0;
				if( _showTime ) {
					fMilliseconds = frame.GetElapsedMillisecs();
				}
				long memoryDelta = 0;
				if( _showMemory ) {
					memoryDelta = frame.GetMemoryDelta();
				}
				
				StringBuilder stringBuilder = new StringBuilder( 50 );
				if( _showBrackets ) {
					stringBuilder.Append( '}' );
				}

				if( _showTime || _showMemory ) {
					stringBuilder.Append( "  [" );
				}

				if( _showTime ) {
					stringBuilder.Append( " " );
					if( fMilliseconds < 1000 ) {
						stringBuilder.Append( fMilliseconds );
						stringBuilder.Append( " ms" );
					}
					else if( fMilliseconds < 60000 ) {
						stringBuilder.Append( fMilliseconds/1000 );
						stringBuilder.Append( " secs" );
					}
					else {
						stringBuilder.Append( fMilliseconds/60000 );
						stringBuilder.Append( " mins" );
					}
					stringBuilder.Append( " " );
				}

				if( _showMemory ) {
					stringBuilder.Append( " " );
					if( Math.Abs( memoryDelta ) < 1000 ) {
						stringBuilder.Append( memoryDelta );
						stringBuilder.Append( " Bytes" );
					}
					else if( Math.Abs( memoryDelta ) < 1000000 ) {
						stringBuilder.Append( ((int)memoryDelta/10)/100f );
						stringBuilder.Append( " KB" );
					}
					else {
						stringBuilder.Append( ((int)memoryDelta/10000)/100f );
						stringBuilder.Append( " MB" );
					}
					stringBuilder.Append( " " );
				}

				if( _showTime || _showMemory ) {
					stringBuilder.Append( "]" );
				}

				return stringBuilder.ToString();
			}
		}

		//=======================================================================================

	}
}
