using System;
using System.Diagnostics;

namespace Exocortex
{
	/// <summary>
	/// Summary description for Random2.
	/// </summary>
	public class Random2
	{

		private Random2() {}
		
		//===============================================================================

		static private Random	_random = new Random();

		//===============================================================================
		
		static public int		RandomInt32() {
			return	_random.Next();
		}
		static public int		RandomInt32( int maxValue ) {
			Debug.Assert( 0 < maxValue );
			return	_random.Next( maxValue );
		}
		static public int		RandomInt32( int minValue, int maxValue ) {
			Debug.Assert( 0 <= minValue );
			Debug.Assert( minValue < maxValue );
			return	_random.Next( maxValue - minValue ) + minValue;
		}

		//-------------------------------------------------------------------------------
		
		static public long		RandomInt64() {
			// each value from _random.Next() is only 32-1=31 bits of randomness
			//   thus to fill 64-1=63 we need 31 bits + 31 bits + 1 bit
			long	alpha	= (long) _random.Next();
			long	beta	= (long) _random.Next();
			return	( alpha << 32 ) + ( beta << 1 ) + _random.Next( 2 );
		}
		static public long		RandomInt64( long maxValue ) {
			Debug.Assert( 0 < maxValue );
			return	Random2.RandomInt64() % maxValue;
		}
		static public long		RandomInt64( long minValue, long maxValue ) {
			Debug.Assert( 0 <= minValue );
			Debug.Assert( minValue < maxValue );
			return	Random2.RandomInt64( maxValue - minValue ) + minValue;
		}

		//-------------------------------------------------------------------------------

		static public float		RandomSingle() {
			return	(float) _random.NextDouble();
		}
		static public float		RandomSingle( float maxValue ) {
			Debug.Assert( 0 < maxValue );
			return	((float) _random.NextDouble() ) * maxValue;
		}
		static public float		RandomSingle( float minValue, float maxValue ) {
			Debug.Assert( 0 <= minValue );
			Debug.Assert( minValue < maxValue );
			return	((float) _random.NextDouble() ) * ( maxValue - minValue ) + minValue;
		}

		//-------------------------------------------------------------------------------
		
		static public double	RandomDouble() {
			return	_random.NextDouble();
		}
		static public double	RandomDouble( double maxValue ) {
			Debug.Assert( 0 < maxValue );
			return	_random.NextDouble() * maxValue;
		}
		static public double	RandomDouble( double minValue, double maxValue ) {
			Debug.Assert( 0 <= minValue );
			Debug.Assert( minValue < maxValue );
			return	_random.NextDouble() * ( maxValue - minValue ) + minValue;
		}
		
		//===============================================================================
		
		// Generates numbers from a gaussian dist with mean = 0, std = 1
		static public double GaussRand() {
			double x1, x2, w;

			do {
				x1 = (2.0 * _random.NextDouble() ) - 1;
				x2 = (2.0 * _random.NextDouble() ) - 1;
				w = x1*x1 + x2*x2;
			}
			while (w >= 1.0);

			w = Math.Sqrt(-2*Math.Log(w)/w);
			return(w*x2);
		}

		//===============================================================================

	}
}
