using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Reflection;

namespace Exocortex.Windows.Forms {

	public class ResourceLoader {

		//--------------------------------------------------------------------------------

		static public bool		IsStream( Type type, string fileName ) {
			string fullName = GetFullResourceName( type, fileName );
			Stream stream = type.Assembly.GetManifestResourceStream( fullName );
			if( stream == null ) {
				return	false;
			}
			stream.Close();
			return	true;
		}

		static public Stream	GetStream( Type type, string fileName ) {
			string fullName = GetFullResourceName( type, fileName );
			Stream stream = type.Assembly.GetManifestResourceStream( fullName );
			if( stream == null ) {
				Debug.WriteLine( "Can not find resource: '" + fullName + "'  ('" + fileName + "')" );
				OutputAvailableResources( type );
				ResourceNotFoundException e = new ResourceNotFoundException( "Can not find resource '" + fullName + "'" );
				Debug.WriteLine( e.ToString() );
				throw e;
			}
			return stream;
		}
		
		static public Icon		GetIcon( Type type, string fileName ) {
			Icon icon = null;

			fileName = "Icons." + fileName;
			
			if( IsResourceCached( type, fileName ) ) {
				//Debug.Write( "!" );
				icon = (Icon) GetCachedResource( type, fileName );
			}
			else {
				//Debug.Write( "." );
				icon = new Icon( GetStream( type, fileName ) );
				AddResourceToCache( type, fileName, icon );
			}

			return icon;
		}

		static public Bitmap	GetBitmap( Type type, string fileName ) {
			Bitmap bitmap = null;

			fileName = "Bitmaps." + fileName;
			
			if( IsResourceCached( type, fileName ) ) {
				//Debug.Write( "!" );
				bitmap = (Bitmap) GetCachedResource( type, fileName );
			}
			else {
				//Debug.Write( "." );
				bitmap = new Bitmap( GetStream( type, fileName ) );
				AddResourceToCache( type, fileName, bitmap );
			}

			return bitmap;
		}

		//--------------------------------------------------------------------------------

		static protected Hashtable _resourceCache = new Hashtable();
		static protected void AddResourceToCache( Type type, string fileName, object resource ) {
			string fullFileName = GetFullResourceName( type, fileName );
			Debug.Assert( _resourceCache.Contains( fullFileName ) == false );
			_resourceCache.Add( fullFileName, resource );
		}
		static protected bool IsResourceCached( Type type, string fileName ) {
			string fullFileName = GetFullResourceName( type, fileName );
			return	_resourceCache.Contains( fullFileName );
		}
		static protected object	GetCachedResource( Type type, string fileName ) {
			string fullFileName = GetFullResourceName( type, fileName );
			Debug.Assert( _resourceCache.Contains( fullFileName ) == true );
			return	_resourceCache[ fullFileName ];
		}

		static protected void ClearResourceCache() {
			_resourceCache.Clear();
		}

		
		//--------------------------------------------------------------------------------

		static protected string	GetFullResourceName( Type type, string fileName ) {
			return type.Namespace + "." + fileName;
		}

		static protected void OutputAvailableResources( Type type ) {
			Debug.WriteLine( "Available resources:" );
			Debug.Indent();
			string[] resourceNames = type.Assembly.GetManifestResourceNames();
			Array.Sort( resourceNames );
			foreach( string name in resourceNames ) {
				Debug.WriteLine( "'" + name + "'" );
			}
			Debug.Unindent();
		}

		//--------------------------------------------------------------------------------

	}
}
